.flow-target-item[data-v-6fafc360]{display:flex;align-items:center;gap:8px;background:var(--color-card-bg-dark);border-radius:var(--radius-m);padding:0 12px;width:fit-content}.flow-target-item[data-v-6fafc360]{display:flex;align-items:center;gap:8px;background:var(--color-card-bg-dark);border-radius:var(--radius-m);padding:0 12px}.flow-icon[data-v-6fafc360]{width:16px;height:16px}.target-node-name[data-v-6fafc360]{font-weight:500}.operation-hint[data-v-f445d7ea]{display:flex;align-items:center;gap:4px;color:var(--color-text-3);font-size:.875rem}.operation-hint .operation-hint-icon[data-v-f445d7ea]{width:16px;height:16px;flex-shrink:0}.node-detail-form-wrapper[data-v-c0f77fbd]{flex:1;overflow-y:auto;padding:8px 12px}.target-list-wrapper[data-v-c0f77fbd]{display:flex;flex-direction:column}.operation-hint[data-v-c0f77fbd]{margin-top:20px;margin-bottom:12px}.record-group[data-v-c0f77fbd]{margin-top:32px}.strong-collapse-title[data-v-c0f77fbd]{font-size:.875rem;font-weight:600;color:var(--color-primary)}.record-parts[data-v-c0f77fbd]{display:flex;flex-direction:column;gap:12px}.record-part-title[data-v-c0f77fbd]{font-size:.75rem;font-weight:600;margin-bottom:4px;color:var(--color-text-2)}.code-viewer[data-v-c0f77fbd]{padding:4px 12px;border-radius:var(--radius-m);background:var(--color-card-bg-dark)}.node-detail-wrapper[data-v-4e2a2e81]{flex:1;overflow-y:auto;padding:12px 16px;display:flex;flex-direction:column;gap:12px}.node-detail-title[data-v-4e2a2e81]{font-size:1rem;font-weight:700}.handle-wrapper[data-v-4e2a2e81]{display:flex;flex:1;align-items:start;gap:20px}.handle-content-wrapper[data-v-4e2a2e81]{width:100%}.delete-icon[data-v-4e2a2e81]{width:1rem;height:1rem;color:var(--color-danger);cursor:pointer;margin-top:8px;flex-shrink:0}.flow-target-list[data-v-4e2a2e81]{display:flex;flex-direction:column;width:100%;gap:8px}
