.el-rate{--el-rate-height:20px;--el-rate-font-size:var(--el-font-size-base);--el-rate-icon-size:18px;--el-rate-icon-margin:6px;--el-rate-void-color:var(--el-border-color-darker);--el-rate-fill-color:#f7ba2a;--el-rate-disabled-void-color:var(--el-fill-color);--el-rate-text-color:var(--el-text-color-primary);--el-rate-outline-color:var(--el-color-primary-light-5);align-items:center;display:inline-flex;height:32px}.el-rate:active,.el-rate:focus{outline:none}.el-rate:focus-visible .el-rate__item .el-rate__icon.is-focus-visible{outline:2px solid var(--el-rate-outline-color);transition:outline-offset 0s,outline 0s}.el-rate__item{color:var(--el-rate-void-color);cursor:pointer;display:inline-block;font-size:0;line-height:normal;position:relative;vertical-align:middle}.el-rate .el-rate__icon{display:inline-block;font-size:var(--el-rate-icon-size);margin-right:var(--el-rate-icon-margin);position:relative;transition:var(--el-transition-duration)}.el-rate .el-rate__icon.hover{transform:scale(1.15)}.el-rate .el-rate__icon .path2{left:0;position:absolute;top:0}.el-rate .el-rate__icon.is-active{color:var(--el-rate-fill-color)}.el-rate__decimal{color:var(--el-rate-fill-color);display:inline-block;overflow:hidden}.el-rate__decimal,.el-rate__decimal--box{left:0;position:absolute;top:0}.el-rate__text{color:var(--el-rate-text-color);font-size:var(--el-rate-font-size);vertical-align:middle}.el-rate--large{height:40px}.el-rate--small{height:24px}.el-rate--small .el-rate__icon{font-size:14px}.el-rate.is-disabled .el-rate__item{color:var(--el-rate-disabled-void-color);cursor:auto}.feedback-item-card[data-v-a25b1194]{display:flex;flex-direction:column;gap:4px;border-radius:var(--radius-m);transition:all .2s}.feedback-time[data-v-a25b1194]{font-size:.75rem;color:var(--color-text-4);flex-shrink:0}.feedback-comment .full-comment[data-v-a25b1194]{margin-top:4px}.feedback-comment:open~.collapsed-comment[data-v-a25b1194]{display:none}.collapsed-comment[data-v-a25b1194]{display:-webkit-box;line-clamp:2;-webkit-line-clamp:2;-webkit-box-orient:vertical;overflow:hidden;text-overflow:ellipsis}.feedback-meta[data-v-a25b1194]{margin-top:8px}.detail-item-title[data-v-a25b1194]{font-size:.75rem;cursor:pointer;color:var(--color-text-3)}.feedback-meta-content[data-v-a25b1194]{font-size:.875rem;padding:8px;margin-top:4px;border-radius:var(--radius-m);background:var(--color-card-bg-dark)}.trace-list-area[data-v-4fb29a7e]{width:100%;flex:1;overflow-y:auto}.trace-list[data-v-4fb29a7e]{display:flex;flex-direction:column;padding:12px 16px}.empty-state[data-v-4fb29a7e]{width:100%;align-self:center}.trace-item-divider[data-v-4fb29a7e]{margin:16px 0}
